""" Django Scaffold Tools

Helps to setup DRF to be ready with few lines of code,
with easy convention configuration of serializers and viewsets.
"""
from .version import VERSION

__version__ = VERSION
