# Generated by Django 3.0.8 on 2020-08-03 21:21

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Attachment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, default='', max_length=255, verbose_name='名称')),
                ('ext_url', models.URLField(blank=True, default='', verbose_name='外部附件链接')),
                ('file', models.FileField(blank=True, upload_to='attachment/', verbose_name='上传附件')),
            ],
            options={
                'verbose_name': '附件',
                'verbose_name_plural': '附件',
                'db_table': 'base_media_attachment',
            },
        ),
        migrations.CreateModel(
            name='Audio',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, default='', max_length=255, verbose_name='名称')),
                ('ext_url', models.URLField(blank=True, default='', verbose_name='外部附件链接')),
                ('audio', models.FileField(upload_to='audio/', verbose_name='音频')),
                ('is_active', models.BooleanField(default=True, verbose_name='是否可用')),
            ],
            options={
                'verbose_name': '音频',
                'verbose_name_plural': '音频',
                'db_table': 'base_media_audio',
            },
        ),
        migrations.CreateModel(
            name='Image',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, default='', max_length=255, verbose_name='名称')),
                ('ext_url', models.URLField(blank=True, default='', verbose_name='外部附件链接')),
                ('image', models.ImageField(blank=True, upload_to='images/', verbose_name='图片')),
            ],
            options={
                'verbose_name': '图片',
                'verbose_name_plural': '图片',
                'db_table': 'base_media_image',
            },
        ),
        migrations.CreateModel(
            name='Video',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, default='', max_length=255, verbose_name='名称')),
                ('ext_url', models.URLField(blank=True, default='', verbose_name='外部附件链接')),
                ('video', models.FileField(upload_to='video/', verbose_name='视频')),
            ],
            options={
                'verbose_name': '视频',
                'verbose_name_plural': '视频',
                'db_table': 'base_media_video',
            },
        ),
    ]
